@@ -47,18 +47,20 @@ proc pgpstate { } {
 # Return: 0 if succeeded; 1 if failed                                #
 #--------------------------------------------------------------------#
 proc enc_pgp_get_key_list {win aid} {
-  global no_of_keys user_id key_id sig_id ldata env
+  global no_of_keys e_user_id e_key_id e_sig_id ldata env
 
   if { [ get_pgppath ] == 1 } {
     return 1
   }
 
-  set tclcmd [ list exec pgp -kv $env(PGPPATH)/pubring.pgp]
+# view the keys on the public keyring
+
+  set tclcmd [ list exec pgp -kv $env(PGPPATH)/pubring.pgp ]
   catch $tclcmd keylist
   pgp_InterpretOutput $keylist pgpresult 1
 
   if {$pgpresult(ok) != 1} {
-    timedmsgpopup "PGP problem" "Couldn't view $env(PGPPATH)/$keyring" 10000
+    timedmsgpopup "PGP problem" "Couldn't view $env(PGPPATH)/pubring.pgp" 10000
     return 1
   }
 
@@ -71,9 +73,9 @@ proc enc_pgp_get_key_list {win aid} {
   foreach line $keylist {
     if [regexp {^(pub) +[0-9]+/([0-9A-F]+) [0-9/]+ (.*)$} \
         $line {} sigid keyid userid] {
-      set user_id(pgp,$i) $userid
-      set  key_id(pgp,$i) $keyid
-      set  sig_id(pgp,$i) $sigid
+      set e_user_id(pgp,$i) $userid
+      set e_key_id(pgp,$i) $keyid
+      set e_sig_id(pgp,$i) $sigid
       if { [string length $userid] > 60 } {
         set dotdot "..."
       } else {
@@ -92,19 +94,19 @@ proc enc_pgp_get_key_list {win aid} {
      %W selection set \[%W nearest %y\];\
      [ format { 
        global no_of_keys
-       global  user_id key_id
+       global  e_user_id e_key_id
        set selkey [%s.enc.keys.lb curselection]
        %s.enc.keys.lb  get [lindex $selkey 0]
        set pgpkey [lindex $selkey 0]
-       set user_id(pgp,enc_cur_key_sel) $user_id(pgp,$pgpkey)
-       set key_id(pgp,enc_cur_key_sel) $key_id(pgp,$pgpkey)
+       set user_id(pgp,enc_cur_key_sel) $e_user_id(pgp,$pgpkey)
+       set key_id(pgp,enc_cur_key_sel) $e_key_id(pgp,$pgpkey)
      } $win $win $win $win] "
  
   if {([string compare $aid "new"]!=0)&&($ldata($aid,enc_asym_keyid)!="")} {
     for {set i 0} {$i < $no_of_keys} {incr i} {
-      if {[string compare $ldata($aid,enc_asym_keyid) $key_id(pgp,$i)] == 0} {
+      if {[string compare $ldata($aid,enc_asym_keyid) $e_key_id(pgp,$i)] == 0} {
         $win.enc.keys.lb  selection set $i
-        set user_id(pgp,enc_cur_key_sel) $user_id(pgp,$i)
+        set user_id(pgp,enc_cur_key_sel) $e_user_id(pgp,$i)
         break
       }
     }
@@ -373,7 +375,7 @@ proc pgp_create_signature {irand} {
 # set up the cmd for signing: 
 # pgp -sb -z pwd +batchmode=on +verbose=0 -u keyid txt_file -o sig_file
 
-  set tclcmd [ list exec pgp -sb -z $asympass +batchmode=on +verbose=0 -u ]
+  set tclcmd [ list exec pgp -sb -z $asympass +batchmode=on -u ]
   set tclcmd [ concat $tclcmd 0x$key_id(pgp,auth_cur_key_sel) ]
   set tclcmd [ concat $tclcmd [ list $local_authtxt_file -o $local_authsig_file ] ]
 
@@ -399,7 +401,7 @@ proc pgp_create_signature {irand} {
 # set up the cmd for extracting the key to a keyfile:
 # pgp -kx +batchmode=on +verbose=0 key_id keyfile
     
-  set tclcmd [ list exec pgp -kx +batchmode=on +verbose=0 ]
+  set tclcmd [ list exec pgp -kx +batchmode=on ]
   set tclcmd [ concat $tclcmd 0x$key_id(pgp,auth_cur_key_sel) ]
   set tclcmd [ concat $tclcmd $local_authkey_file ]
 
@@ -791,7 +793,7 @@ proc pgp_InterpretOutput { in outvar key} {
 	putlogfile "   pgp_InterpretOutput: USER = $pgpresult(userid) "
      } else {
        if { $key != 1 } {
-	 set pgpresult(userid) $key
+	 set pgpresult(userid) \"$key\"
        } else {
 	 set pgpresult(userid) "none"
        }
@@ -805,6 +807,7 @@ proc pgp_InterpretOutput { in outvar key} {
        set tclcmdkey [ concat $tclcmdkey $pgpresult(userid) ]
        putlogfile "  pgp_InterpretOutput: $tclcmdkey"
        set resultkey [ catch $tclcmdkey output1 ]
+       putlogfile "output1 = \n$output1"
        set keyinfo   [split $output1 "\n"]
        set i  0
 
@@ -1060,7 +1063,7 @@ proc pgp_GetPass { key } {
 
         while 1 {
 
-          set result [catch {Misc_GetPass "Enter PGP passphrase" "You have received an announcement encrypted for $key.\n\nA passphrase is needed to unlock the required secret key.\n\n(Your passphrase may be compromised if SDR is not running\nlocally and a secure connection s not being used)"} passw ]
+          set result [catch {Misc_GetPass "Enter PGP passphrase" "You have received an announcement encrypted for $key.\n\nA passphrase is needed to unlock the required secret key.\n\n(Your passphrase may be compromised if SDR is not running\nlocally and a secure connection is not being used)"} passw ]
 
           putlogfile "pgp_GetPass - result is >$result< and passw is >$passw<"
 
@@ -1095,7 +1098,7 @@ proc pgp_GetPass { key } {
 
     while 1 {
 
-      set result [catch {Misc_GetPass "Enter PGP passphrase" "You have received an announcement encrypted for $key.\n\nA passphrase is needed to unlock the required secret key.\n\n(Your passphrase may be compromised if SDR is not running\nlocally and a secure connection s not being used)"} passw ]
+      set result [catch {Misc_GetPass "Enter PGP passphrase" "You have received an announcement encrypted for $key.\n\nA passphrase is needed to unlock the required secret key.\n\n(Your passphrase may be compromised if SDR is not running\nlocally and a secure connection is not being used)"} passw ]
 
       putlogfile "pgp_GetPass - result is >$result< and passw is >$passw<"
 
@@ -1399,468 +1402,9 @@ proc pgp_AddCert { filename title label } {
          return 0 
        }
 }
-proc pgp_smart { title label but} {
-    global getsmartans env
 
-    if { [ get_pgppath ] == 1 } {
-      return 1
-    }
-
-     set w .getsmartans
-     catch {destroy $w}
-     sdr_toplevel "$w -borderwidth 10" $title
-     label $w.lab -text  $label
-     pack $w.lab -side top -fill x -expand true
-     frame $w.but
-     pack $w.but -side top -fill x -expand true
-     button $w.but.yes -text $but -command {set getsmartans(yes) 1 }
-     button $w.but.no -text NO -command { set getsmartans(yes) 0} 
-      pack $w.but.yes -side left -fill x -expand true
-      pack $w.but.no -side left -fill x -expand true
-      foreach f [list $w.but.yes $w.but.no] {
-        bindtags $f [list .getsmartans [winfo class $f] $f all]
-          }
-        bind .getsmartans <Alt-o> "focus $w.but.yes ; break"
-        bind .getsmartans <Alt-c> "focus $w.but.no ; break"
-        bind .getsmartans <Alt-Key>  break
-        bind .getsmartans <Return> { set getsmartans(yes) 1}
-        bind .getsmartans <Control-c> { set getsmartans(yes) 0}
-       grab $w
-       tkwait variable getsmartans(yes)
-       grab release $w
-       destroy $w
-       if { $getsmartans(yes) == 1} {
-        return 1
-       } else {
-         return 0 
-       }
-}
-proc Help_asym {help} {
-    global sdr
-    set sdrversion 2.5
-    set label "Key Generation"
-    set local_help_file "[glob -nocomplain [resource sdrHome]]/$help.txt"
-
-    if [Help_Toplevel .pgphelp "Key Generation Help" pgpHelp] {
-        Help1_Label .pgphelp.but label {left fill} -text $label
-        Help1_AddBut .pgphelp.but setup "Make PGP Key" [list pgp_Setup]
-        Help1_AddBut .pgphelp.but setup1 "Make X509 Key" [list x509_Setup]
-        Help1_AddBut .pgphelp.but setup2 "Make Des Key" [list Des_Setup]
-        set help "
-
-For Asymmetric encryption and authentication we use Pretty Good Privacy
-(PGP) and Public Key Cryptographic System (PKCS7) and for symetric
-encryption DES.
-
-Pretty Good Privacy (tm) (PGP), from Network Associates, is a high
-security cryptographic software application for MSDOS, Unix, VAX/VMS, and
-other computers.  PGP allows people to exchange files or messages with
-privacy, authentication, and convenience. PGP is based on public key
-cryptography. PGP combines the convenience of the Rivest-Shamir-Adleman
-(RSA) public key cryptosystem with the speed of symmetric cryptography.
-It uses message digests for digital signatures, data compression before
-encryption, good ergonomic design, and sophisticated key management.
-PGP uses \"message digests\" to form signatures.  A message digest is a
-128-bit cryptographically strong one-way hash function of the message. 
-It is somewhat analogous to a \"checksum\" or CRC error checking code, in
-that it compactly \"represents\" the message and is used to detect
-changes in the message.  Unlike a CRC, however, it is computationally
-infeasible for an attacker to devise a substitute message that would
-produce an identical message digest.  The message digest gets encrypted
-by the private key to form a signature.  Documents are signed by
-prefixing them with signature certificates, which contain the key ID of
-the key that was used to sign it, a private-key-signed message digest
-of the document, and a timestamp of when the signature was made.  The
-receiver to look up the sender's public key to check the signature uses
-the key ID.  The receiver's software automatically looks up the
-sender's public key and user ID in the receiver's public key ring. The
-key ID of the public key used to encrypt them prefixes encrypted
-files.  The receiver uses this key ID message prefix to look up the
-private key needed to decrypt the message.  The receiver's software
-automatically looks up the necessary private decryption key in the
-receiver's private key ring. 
-
-Public Key Cryptographic System (PKCS7): There are many security
-toolkits. Our implementation of SDR v1.5 uses Secude for generating
-X509 keys and encryption and authentication. The Secude development kit
-is a library that offers well-known and established symmetric and
-asymmetric cryptography for popular hardware and operating system
-platforms. The development kit consists of a set of functions which
-allows the incorporation of security in practically any application (e.g.
-client/server, e-mail, office applications) and documentation in
-Hypertext Markup Language (HTML) which describes in detail the C
-programming interface. There are also various commands collected in a
-security command shell to ensure an immediate deployment of security. 
-
-Before you generate keys you need to get an e-mail system which has the
-capability of sending information securely; for example Exmh can send
-encrypted and authenticated text body part using PGP. Eudora can send PGP
-and S-MIME authenticated and encrypted messages. Next you need to
-establish a group membership, possibly with an e-mail list. Section 3.4.1
-of \"USER GUIDE\" shows the step required to generate DES, X509 and PGP
-keys and use your chosen mail system to send it to the group members.
-Section 3.4.2 of \"USER GUIDE\" will show you how to store keys recieved
-via E-mail to be used by SDR v2.5. "
-
-   
-        Help1_Text .pgphelp $help
-    }
-}
-proc Help1_AddBut {par but txt cmd {where {right padx 1}} } {
-    # Create a Packed button.  Return the button pathname
-    set cmd2 [list button $par.$but -text $txt -command $cmd]
-    if [catch $cmd2 t] {
-        catch {puts stderr "Help1_AddBut (warning) $t"}
-        eval $cmd2 {-font fixed}
-    }
-    pack append $par $par.$but $where
-    return $par.$but
-}
-proc Help_Toplevel { path name {class Dialog} {dismiss yes}} {
-    global exwin
-    if [catch {wm state $path} state] {
-        set t [Help1_Toplevel $path $name $class]
-        if ![info exists exwin(toplevels)] {
-            set exwin(toplevels) [option get . exwinPaths {}]
-        }
-        set ix [lsearch $exwin(toplevels) $t]
-        if {$ix < 0} {
-            lappend exwin(toplevels) $t
-        }
-        if {$dismiss == "yes"} {
-            set f [Help1_Frame $t but Menubar {top fill}]
-            Help1_AddBut $f quit "Dismiss" [list Help1_Dismiss $path]
-        }
-        return 1
-    } else {
-        if {$state != "normal"} {
-            catch {
-                wm geometry $path $exwin(geometry,$path)
-                #putlogfile "Help_Toplevel $path $exwin(geometry,$path)"
-            }
-            wm deiconify $path
-        } else {
-            catch {raise $path}
-        }
-        return 0
-    }
-}
-proc Help1_Label { frame {name label} {where {left fill}} args} {
-    set cmd [list label $frame.$name ]
-    if [catch [concat $cmd $args] t] {
-        #putlogfile "Help1_Label (warning) $t"
-        eval $cmd $args {-font fixed}
-    }
-    pack append $frame $frame.$name $where
-    return $frame.$name
-}
-proc Help1_Frame {par child {class Sdr} {where {top expand fill}} args } {
- 
-    if {$par == "."} {
-        set self .$child
-    } else {
-        set self $par.$child
-    }
-    eval {frame $self -class $class} $args
-    pack append $par $self $where
-    return $self
-}
-proc Help1_Dismiss { path {geo ok} } {
-    global exwin
-    case $geo {
-        "ok" {
-            set exwin(geometry,$path) [wm geometry $path]
-        }
-        "nosize" {
-            set exwin(geometry,$path) [string trimleft [wm geometry $path] 0
-123456789x]
-        }
-        default {
-            catch {unset exwin(geometry,$path)}
-        }
-    }
-    if [info exists exwin(geometry,$path)] {
-        # Some window managers return geometry like
-        # 80x24+-1152+10
-        regsub -all {\+-} $exwin(geometry,$path) + exwin(geometry,$path)
-    }
-    #Sdr_Focus
-    wm withdraw $path
-    update idletasks    ;# Helps window dismiss
-}
 proc Sdr_Focus {} {
     global exwin
     focus $exwin(mtext)
 }
 
-
-
-
-proc Help1_Toplevel { path name {class Dialog} {x {}} {y {}} } {
-    set self [toplevel $path -class $class]
-    set usergeo [option get $path position Position]
-    if {$usergeo != {}} {
-        if [catch {wm geometry $self $usergeo} err] {
-            putlogfile "Help_Toplevel $self $usergeo => $err"
-        }
-    } else {
-        if {($x != {}) && ($y != {})} {
-            putlogfile "Event position $self +$x+$y"
-            wm geometry $self +$x+$y
-        }
-    }
-    wm title $self $name
-    wm iconname $self $name
-    wm group $self .
-    return $self
-}
-
-
-proc Help1_Text {frame help} {
-    # Create the text widget used to display messages
-    set local_help_file "[glob -nocomplain [resource sdrHome]]/$help.txt"
-    global exwin
-        set side right
-    set t [text $frame.lb -setgrid true -width 40 -height 15  \
-                -yscroll "$frame.sb set" -relief \
-                flat -wrap none \
-              -selectforeground [resource activeForeground] \
-              -highlightthickness 0  ]
-               scrollbar $frame.sb -command "$frame.lb yview" \
-                -background [resource scrollbarForeground] \
-                -troughcolor [resource scrollbarBackground] \
-                -borderwidth 1 -relief flat \
-                -highlightthickness 0
-        pack $frame.lb -side left -fill both -expand true
-        pack $frame.sb -side right -fill y
-        $t insert insert $help
-       # if [catch {open $local_help_file r} in] {
-       #     $t insert insert "Cannot find file pgp.txt to display"
-       #     $t configure -state disabled
-       # } else {
-       #     $t insert insert [read $in]
-       # }
-}
-
-
-proc pgp_Setup {  } {
-    global pgp env
- 
-    if { [ get_pgppath ] == 1 } {
-      return 1
-    }
-
-    set PGPPATH $env(PGPPATH)
-    set pgp(pgppath) $env(PGPPATH)
- 
-    # make the key pair and self sign it
-   # exec xterm -title "PGP Setup" -e sh -c {
-   #     cd ${PGPPATH}
-   #     rm -f pubring.bak
-   #     pgp -kg
-   #     rm -f pubring.bak
-   #     pgp +verbose=0 +force=on -ks "" -u ""
-   # } >& /dev/console
-    exec xterm -title "PGP Setup" -e sh -c {
-        cd ${PGPPATH}
-        rm -f pubring.bak
-        pgp -kg
-        rm -f pubring.bak
-        pgp +verbose=0 +force=on -ks "" -u ""
-    }
- 
-    #if {![file exists "$env(PGPPATH)/pubring.bak"]} {
-        #return
-    #}
- 
-        set pgp(secring) $pgp(pgppath)/secring.pgp
-        set pgp(privatekeys) [pgpExec_KeyList "" $pgp(secring)]
-        #putlogfile "$pgp(privatekeys)"
- 
-    # send the key to the keyservers
-    set pgpfile [pgpExec_GetKeys [lindex [lindex $pgp(privatekeys) 0] 0] ]
-    if  [info exists env(MAILAGENT] {
-    set mailagent $env(MAILAGENT)
-    } else {
-    set mailagent [Misc_Gettext "MAILAGENT" "Please enter mail agent"]
-    }
-    
-    if { $mailagent != 0 } {
-    set tclcmd [ list exec $mailagent ]
-
-    set result [ catch $tclcmd output ]
-    file delete $pgpfile
-   }
-}
-proc pgpExec_KeyList { pattern keyring } {
- 
-    set pattern [string trimleft $pattern "<>|2"]
-    pgpExec_Batch [list -kv $pattern $keyring] keylist {}
- 
-    # drop the revoked keys
-    regsub -all "\n(pub|sec) \[^\n]+\\*\\*\\* KEY REVOKED \\*\\*\\*(\n\[\t ]
-\[^\n]+)+" $keylist "" keylist
- 
-    if { ![regexp {.*(pub|sec) +[0-9]+(/| +)([0-9A-F]+) +[0-9]+/ ?[0-9]+/ ?[
-0-9]+ +(.*)} $keylist]} {
-        return {}
-    } else {
-        set keylist [split $keylist "\n"]
-        set keys {}
-        set key {}
-        foreach line $keylist {
-            if [regexp {^ *(pub|sec) +[0-9]+(/| +)([0-9A-F]+) +[0-9]+/ ?[0-9
-]+/[0-9]+ +(.*)$} $line {} {} {} keyid userid] {
-                #set key [list "0x$keyid" [string trim $userid]]
-                set key [list [string trim $userid]]
-                lappend keys $key
-            }
-        }
-        return $keys
-    }
-}
-proc pgpExec_GetKeys { key } {
-    global  env
-    set pgpfile  "[glob -nocomplain [resource sdrHome]]/pgpkeyfile"
-
-    if { [ get_pgppath ] == 1 } {
-      return 1
-    }
-
-    set tmpfile  "[glob -nocomplain [resource sdrHome]]/tmpfile"
-            set p [pgp_GetPass $key]
-            if {[string length $p] == 0} {
-                return 0
-            }
-           set fileid [open $pgpfile w]
-            puts $fileid [concat "password: " $p]
-            
-            close $fileid
-	set tclcmd [ list exec pgp -akx +armorlines=0 +batchmode=on $key $tmpfile  $env(PGPPATH)/secring.pgp]
-            set result [ catch $tclcmd output ]
-        if {![file exists $tmpfile] && [file exists "$tmpfile.asc"]} {
-            pgp_Rename "$tmpfile.asc" $tmpfile
-        }
-
-            set tmpid [open $tmpfile r]
-            set content [read $tmpid]
-            close $tmpid
-             file delete $tmpfile
-            set fileid [open $pgpfile a]
-            puts $fileid  $content
-             close $fileid
-        set tclcmd [ list exec pgp -akx +armorlines=0 +batchmode=on $key $tmpfile  $env(PGPPATH)/pubring.pgp]
-            set result [ catch $tclcmd output ]
-        if {![file exists $tmpfile] && [file exists "$tmpfile.asc"]} {
-            pgp_Rename "$tmpfile.asc" $tmpfile
-        }
-            set tmpid [open $tmpfile r]
-            set content [read $tmpid]
-            close $tmpid
-             file delete $tmpfile
-            set fileid [open $pgpfile a]
-            puts $fileid $content
-            close $fileid
-return $pgpfile
-}
-proc Des_Setup {  } {
-    global  env deskey
-    set result [creat_des_key]
-    #putlogfile "$result"
-    if { $result == 1 } {
-    set desfile  "[glob -nocomplain [resource sdrHome]]/deskeyfile"
-    set i 0
-    while { $i == 0 } {
-    if {[string length $deskey(pass)]<8} {
-        bell
-        timedmsgpopup "Encryption keys must be at least 8 characters" "TRY again" 3000
-        Des_Setup
-    } else {
-    if {[string compare $deskey(pass) $deskey(pass1)] !=0} {
-        timedmsgpopup "You have not entered the same key twice" "TRY again" 3000
-         Des_Setup
-    } else {
-        set i 1
-     }
-    }
-   }
-    set fileid [open $desfile w]
-            puts $fileid [concat "DES Encryption Key: " $deskey(pass)]
-            close $fileid
-     set fileid [open $desfile a]
-            puts $fileid [concat "DES Encryption Group: " $deskey(name)]
-            close $fileid
-
-    if  [info exists env(MAILAGENT] {
-    set mailagent $env(MAILAGENT)
-    } else {
-    set mailagent [Misc_Gettext "MAILAGENT" "Please enter mail agent"]
-    }
-    
-    set tclcmd [ list exec $mailagent ]
-
-    set result [ catch $tclcmd output ]
-    file delete $desfile
-  }
-}
-proc creat_des_key {} {
-    global  env desinfo deskey
-    global desname despass despass1
-    set w .desinfo
-    catch {destroy $w}
-    sdr_toplevel "$w -borderwidth 2" "Des Information to send to group"
-    frame $w.f -borderwidth 5 -relief groove
-    pack $w.f -side top
-    message $w.f.l -aspect 500  -text "Please enter name of encryption group and key"
-    pack $w.f.l -side top
-    frame $w.f.f0
-    pack $w.f.f0 -side top -fill x -expand true
-    label $w.f.f0.l -text "Des Encryption group"
-    pack $w.f.f0.l -side left -anchor e -fill x -expand true
-    entry $w.f.f0.e -width 40 -relief sunken -borderwidth 1 \
-        -bg [option get . entryBackground Sdr] \
-         -highlightthickness 0   -textvariable desname
-    pack $w.f.f0.e -side left
- 
-    frame $w.f.f1
-    pack $w.f.f1 -side top -fill x -expand true
-    label $w.f.f1.l -text "Encryption Key at least 8 character"
-    pack $w.f.f1.l -side left -anchor e -fill x -expand true
-    #entry ..f.f1.e -width 40 -relief sunken -borderwidth 1 \
-#       -bg [option get . entryBackground Sdr] \
-#        -highlightthickness 0
-    password $w.f.f1.e -width 30 -relief sunken -borderwidth 1 \
-        -variable despass -background [option get . entryBackground Sdr]
-    pack $w.f.f1.e -side left
-    frame $w.f.f2
-    pack $w.f.f2 -side top -fill x -expand true
-    label $w.f.f2.l -text "Encryption Key (again)"
-    pack $w.f.f2.l -side left -anchor e -fill x -expand true
-    password $w.f.f2.e -width 30 -relief sunken -borderwidth 1 \
-        -variable despass1 -background [option get . entryBackground Sdr]
-    pack $w.f.f2.e -side left
- 
-    frame $w.f.f3
-    pack $w.f.f3 -side top -fill x -expand true
- 
-    button $w.f.f3.ok -text OK -command {set desinfo(ok) 1 }
-    pack $w.f.f3.ok -side left -fill x -expand true
-    button $w.f.f3.cancel -text Cancel \
-            -command {set desinfo(ok) 0 }
-    pack $w.f.f3.cancel -side left -fill x -expand true
-    grab $w
-       tkwait variable desinfo(ok)
-       #grab release $w
-       destroy .desinfo
-       if { $desinfo(ok) == 1} {
-        set deskey(name) $desname
-        set deskey(pass) $despass
-        set deskey(pass1) $despass1
-       destroy .desinfo
-          return 1
-       } else {
-        destroy .desinfo
-        return 0
-       }
-}
-
