@@ -2,13 +2,13 @@
 source ../src/plugins.tcl
 parse_plugins "../src/plugins" no
 set file [open parsed_plugins.tcl "w"]
-puts $file "set applist \"$applist\""
-puts $file "set medialist \"$medialist\""
-puts $file "set rules \"$rules\""
-puts $file "set createrules \"$createrules\""
+puts $file [list set applist $applist]
+puts $file [list set medialist $medialist]
+puts $file [list set rules $rules]
+puts $file [list set createrules $createrules]
 foreach ary {tooldata mediadata fmts protos protonames fmtnames mappings attrs attrnames attrvaluenames attrflags noattrflags noattrlist defattrlist withattrs macros macrokeys fmtlayers} {
     foreach key [array names $ary] {
-	puts $file "set [set ary]($key) \"[set [set ary]($key)]\""
+	puts $file [list set [set ary]($key) [set [set ary]($key)]]
     }
 }
 close $file
