@@ -644,7 +644,8 @@ proc start_media_tool {aid mnum proto fmt attrlist} {
 	}
 	if {[llength $rulelist]==0} {
 	    # Special case for handling SDP/SAP directory sessions:
-	    if {$media=="directory" && $proto=="SAP" && $fmt=="SDP"} {
+        if {($media=="directory" || $media=="application")
+           && $proto=="SAP" && $fmt=="SDP"} {
 		return [launch_directory $ldata($aid,$mnum,addr) \
 			                 $ldata($aid,$mnum,port) \
                                          $ldata($aid,ttl) \
@@ -718,7 +719,7 @@ proc apply_startup_rule {aid mnum proto fmt rule attrlist} {
 
 # replaced next line as it adds {} around elements of cmd line. New one doesn't
 #   set rule [lrange $rule 1 end]
-    set rule [ string range $rule [string wordend $rule 0] end]
+    set rule [ string range $rule [string first " " $rule] end]
 
     foreach noattr \
 	"$noattrlist($tool.$media.$proto) $noattrlist($tool.$media.$proto.$fmt)" {
