@@ -9,6 +9,9 @@ extern Tcl_Interp *interp;
 extern char hostname[];
 extern char username[];
 extern unsigned long hostaddr;
+#if HAVE_IPv6
+extern struct in6_addr hostaddr_v6;
+#endif
 extern int rxsock[];
 extern int no_of_rx_socks;
 extern int doexit;
@@ -88,14 +91,18 @@ int parse_announcement(int enc, char *data, int length,
   /* if someone else is repeating our announcements, be careful
      not to re-announce their modified version ourselves */
 
+/* 
+ * REVIEW -- this entire funtion not called? If so the 'if' statement
+ * below, won't work with IPv6. -- MM 
+*/
  if (src == hfrom || src != hostaddr) {
     parse_entry(NULL,data,length,src,hfrom,addr,port,sec,"trusted",recvkey ,
                 NULL, NULL, NULL, NULL,NULL, NULL, NULL, NULL,NULL,NULL);
+
   } else {
     parse_entry(NULL,data,length,src,hfrom,addr,port,sec,"untrusted",recvkey,
                 NULL, NULL, NULL, NULL,NULL, NULL, NULL, NULL,NULL,NULL);
   }
-
   return 0;
 }
 
@@ -104,7 +111,7 @@ int parse_announcement(int enc, char *data, int length,
 /* add - sap header, timeout, auth_hdr and enc_hdr                      */
 /* buf malloced in send_advert(), filled here and sent in send_advert() */
 /*----------------------------------------------------------------------*/
-int build_packet(char *buf, char *adstr, int len, int encrypt, 
+int build_packet(char *buf, char *adstr, int addr_fam, int len, int encrypt, 
                  u_int auth_len, u_int hdr_len, 
                  struct auth_info *authinfo, struct priv_header *sapenc_p)
 {
@@ -115,11 +122,14 @@ int build_packet(char *buf, char *adstr, int len, int encrypt,
   int len_add=0;
   int privlen=0;
   int i=0;
+  u_int host;
+  int sap_hdr_len = 0;
 
   writelog(printf(" -- entered build_packet --\n");)
  
 /* set the sap_hdr to point to the start of buf */
-
+     
+  
   bp = (struct sap_header *)buf;
 
 /* set up some basic sap_header fields */
@@ -129,7 +139,23 @@ int build_packet(char *buf, char *adstr, int len, int encrypt,
   bp->compress = 0;
   bp->authlen  = auth_len / 4;
   bp->msgid    = 0;
-  bp->src      = (unsigned long)htonl(hostaddr);
+  
+/* Write the v6 or v4 source address in buf */
+  /* bp->src = (unsigned long)htonl(hostaddr); */
+  if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      memcpy(buf+sizeof(struct sap_header), (char *) &hostaddr_v6, 16);
+      len_add = SAPV6_HDR_LEN;
+      sap_hdr_len = SAPV6_HDR_LEN;
+      bp->addr     = 1;
+#endif
+  } else {
+      host = (unsigned long)htonl(hostaddr);
+      memcpy(buf+sizeof(struct sap_header), (char *) &host, 4);
+      len_add = SAPV4_HDR_LEN;
+      sap_hdr_len = SAPV4_HDR_LEN;
+      bp->addr     = 0;
+  }
 
   if ( (hdr_len == 0) && (encrypt == 0) ) {
     bp->enc = 0;
@@ -137,15 +163,13 @@ int build_packet(char *buf, char *adstr, int len, int encrypt,
     bp->enc = 1;
   }
 
-  len_add += sizeof(struct sap_header);
-
 /* the sap_header has been filled out now so do the auth_header */
 
   if ( auth_len != 0 ) {
 
 /* auth header (length is 2 as Goli has signature length as 2nd byte !) */
 
-    auth_hdr = (struct auth_header *)((char *)buf+sizeof(struct sap_header));
+    auth_hdr = (struct auth_header *)((char *)buf+sap_hdr_len);
 
     auth_hdr->version   = authinfo->version;
     auth_hdr->padding   = authinfo->padding;
@@ -154,7 +178,7 @@ int build_packet(char *buf, char *adstr, int len, int encrypt,
 
 /* add signature    */
 
-    ap = (char *)buf+sizeof(struct sap_header)+AUTH_HEADER_LEN;
+    ap = (char *)buf+ sap_hdr_len+AUTH_HEADER_LEN;
     memcpy(ap, authinfo->signature, authinfo->sig_len);
     ap += authinfo->sig_len;
 
@@ -174,27 +198,27 @@ int build_packet(char *buf, char *adstr, int len, int encrypt,
 /* sap_hdr and auth_hdr filled now so fill the privacy header and data */
 
   if ( (hdr_len == 0) && (encrypt == 0) ) {
-    memcpy(buf+sizeof(struct sap_header)+auth_len, adstr, len);
+    memcpy(buf+sap_hdr_len+auth_len, adstr, len);
   } else {
 
 /* first add the common timeout field - always 0 at the moment */
 
     for (i=0; i<4; i++) {
-      buf[sizeof(struct sap_header)+auth_len+i]=0;
+      buf[sap_hdr_len+auth_len+i]=0;
     }
 
 /* fill in the privacy header */
 
-    privlen = add_privacy_header(buf, auth_len, sapenc_p);
+    privlen = add_privacy_header(buf, auth_len, sapenc_p, addr_fam);
 
 /* fill in the data                                                   */
 /* for DES privlen will be 4 and for asymm it will be the whole thing */
 
     if (sapenc_p == NULL) {
-      memcpy(buf+sizeof(struct sap_header)+auth_len+TIMEOUT+privlen,adstr,len);
+      memcpy(buf+sap_hdr_len+auth_len+TIMEOUT+privlen,adstr,len);
       len_add += TIMEOUT + privlen;
     } else {
-      memcpy(buf+sizeof(struct sap_header)+auth_len+TIMEOUT+ENC_HEADER_LEN,adstr,len);
+      memcpy(buf+sap_hdr_len+auth_len+TIMEOUT+ENC_HEADER_LEN,adstr,len);
       len_add += TIMEOUT + ENC_HEADER_LEN;
     }
 
@@ -210,16 +234,25 @@ int build_packet(char *buf, char *adstr, int len, int encrypt,
 /* add_privacy_header -                                                    */
 /* adds privacy header to symmetrically/asymmetrically encrypted packets   */
 /*-------------------------------------------------------------------------*/
-int add_privacy_header(char *buf, int auth_len, struct priv_header *sapenc_p)
+int add_privacy_header(char *buf, int auth_len, struct priv_header *sapenc_p, int addr_fam)
 {
   struct priv_header *priv_hdr;
   unsigned int padlen;
   int privlen=0;
   char *ap=NULL;
+  int sap_hdr_len=NULL;
 
   writelog(printf(" -- entered add_privacy_header --\n");)
 
-  priv_hdr = (struct priv_header *)((char *)buf+sizeof(struct sap_header)+auth_len+TIMEOUT);
+  if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      sap_hdr_len = SAPV6_HDR_LEN;
+#endif
+  } else {
+      sap_hdr_len = SAPV4_HDR_LEN;
+  }
+
+  priv_hdr = (struct priv_header *)((char *)buf+sap_hdr_len+auth_len+TIMEOUT);
 
 /* the only symmetric encryption at the moment is DES */
 
@@ -235,7 +268,7 @@ int add_privacy_header(char *buf, int auth_len, struct priv_header *sapenc_p)
 
 /* add the padding and padlen in the final two bytes of the hdr */
 
-    ap = (char *)buf+sizeof(struct sap_header)+TIMEOUT+auth_len+ENC_HEADER_LEN;
+    ap = (char *)buf+sap_hdr_len+TIMEOUT+auth_len+ENC_HEADER_LEN;
     ap[0] = 0;
     ap[1] = 2;
 
@@ -309,13 +342,13 @@ int store_data_to_announce(struct advert_data *addata,
   }
 }
 
-/* ---------------------------------------------------------------------- */
-/* ui_createsession - creates the session announcement                    */
-/*                                                                        */
-/* argv[1]=sess; [2]=stoptime; [3]=addr; [4]=port; [5]=ttl; [6]=keyname   */
-/*     [7]=auth_type; [8]=enc_type; [9]=keyid(auth); [10]=keyid(enc)      */
-/*                                                                        */
-/* ---------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------*/
+/* ui_createsession - creates the session announcement                      */
+/*                                                                          */
+/* argv[1]=sess; [2]=stoptime; [3]=sap addr; [4]=port; [5]=ttl; [6]=keyname */
+/*     [7]=auth_type; [8]=enc_type; [9]=keyid(auth); [10]=keyid(enc)        */
+/*                                                                          */
+/* ------------------------------------------------------------------------ */
 int ui_createsession(dummy, interp, argc, argv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
@@ -339,6 +372,30 @@ int ui_createsession(dummy, interp, argc, argv)
   int i, *authinfo=0, *encinfo=0;
   int irand, new_len;
   int rc;
+  int addr_fam = IPv4;
+  int addr_len = IPV4_ADDR_LEN;
+
+  struct in_addr in;
+#ifdef HAVE_IPv6
+  static char source[INET6_ADDRSTRLEN];
+#else
+  static char source[INET_ADDRSTRLEN];
+#endif
+
+/* 
+ * Determine protocol family of session, IPv4 or IPv6, by looking at the
+ * SAP address contained in argv[3].
+ */
+  if (strncmp("ff", argv[3], 2) == 0){
+#ifdef HAVE_IPv6
+      inet_ntop(AF_INET6, &hostaddr_v6, source, INET6_ADDRSTRLEN);
+      addr_fam = IPv6;
+      addr_len = IPV6_ADDR_LEN;
+#endif
+  } else {
+      in.s_addr=htonl(hostaddr);
+	  strncpy(source, inet_ntoa(in), INET_ADDRSTRLEN);
+  }
 
 /* Clear key */
 
@@ -358,6 +415,7 @@ int ui_createsession(dummy, interp, argc, argv)
   port     = atoi(argv[4]);
   interval = INTERVAL;
 
+
 /* need the copy because parse entry splats the data */
 
   strncpy(data, argv[1], MAXADSIZE);
@@ -372,7 +430,8 @@ int ui_createsession(dummy, interp, argc, argv)
 /* create the advert_data structure */
 
     if (addata == NULL) {
-      addata=(struct advert_data *)malloc(sizeof (struct advert_data));
+      addata=(struct advert_data *)malloc(sizeof (struct advert_data)+addr_len);
+      memset(addata, 0, sizeof (struct advert_data)+addr_len);
       addata->sapenc_p = NULL;
       addata->data     = NULL;
       addata->sap_hdr  = NULL;
@@ -412,7 +471,7 @@ int ui_createsession(dummy, interp, argc, argv)
       return TCL_OK;
     } else {
       Tcl_SetVar(interp, "validpassword", "1", TCL_GLOBAL_ONLY);
-      addata=(struct advert_data *)malloc(sizeof (struct advert_data));
+      addata=(struct advert_data *)malloc(sizeof (struct advert_data)+addr_len);
       addata->data=NULL;
       addata->authinfo=(struct auth_info *)malloc(sizeof(struct auth_info));
       strcpy(authstatus, "Authenticated");
@@ -445,7 +504,8 @@ int ui_createsession(dummy, interp, argc, argv)
 /* create the advert_data structure if not created already */
 
     if(addata == NULL) {
-      addata=(struct advert_data *)malloc(sizeof (struct advert_data));
+      addata=(struct advert_data *)malloc(sizeof (struct advert_data)+addr_len);
+      memset(addata, 0, sizeof (struct advert_data)+addr_len);
       addata->data     = NULL;
       addata->authinfo = NULL;
       addata->sap_hdr  = NULL;
@@ -485,7 +545,8 @@ int ui_createsession(dummy, interp, argc, argv)
     }
 
     if(addata == NULL) {
-      addata=(struct advert_data *)malloc(sizeof (struct advert_data));
+      addata=(struct advert_data *)malloc(sizeof (struct advert_data)+addr_len);
+      memset(addata, 0, sizeof (struct advert_data)+addr_len);
       addata->data     = NULL;
       addata->authinfo = NULL;
       addata->sap_hdr  = NULL;
@@ -540,7 +601,7 @@ int ui_createsession(dummy, interp, argc, argv)
 
     if ( strcmp(argv[7],"pgp")==0 ) {
 
-      new_len = gen_new_data(data,new_data,argv[6],addata);
+      new_len = gen_new_data(data,new_data,argv[6],addata,addr_fam);
 
 /* do we need these ? */
 
@@ -574,7 +635,7 @@ int ui_createsession(dummy, interp, argc, argv)
 
 /* X.509 stuff - I haven't looked at this */
 
-    new_len=gen_new_data(data,new_data,argv[6],addata);
+    new_len=gen_new_data(data,new_data,argv[6],addata,addr_fam);
     irand = (lbl_random()&0xffff);
     memset(authmessage,0,AUTHMESSAGELEN);
     memset(authstatus,0,AUTHSTATUSLEN);
@@ -587,16 +648,17 @@ int ui_createsession(dummy, interp, argc, argv)
   }
 
 /* encryption and authentication has been finished so call parse_entry */
-
-    parse_entry(aid, data, strlen(data), hostaddr, hostaddr, argv[3], port,
+    parse_entry(aid, data, strlen(data), source, source, argv[3], port,
          tv.tv_sec, "trusted", key, 
          argv[7], authstatus, authinfo, argv[9], 
          argv[8], encstatus,  encinfo,  argv[10], authmessage,encmessage);
  
 /* queue the ad for sending */
-
-    queue_ad_for_sending(aid, argv[1], interval, endtime, argv[3], port, 
-         ttl, argv[6], argv[7], authstatus ,argv[8], encstatus, addata);
+    /* printf("ui_createsession calling queue_ad_4_sending: addr/port %s/%d\n",
+           argv[3], port); */
+    queue_ad_for_sending(aid, argv[1], interval, endtime, argv[3], addr_fam,
+                         port, ttl, argv[6], argv[7], authstatus ,argv[8], 
+                         encstatus, addata);
 
 /* free up some space */
  
@@ -606,8 +668,8 @@ int ui_createsession(dummy, interp, argc, argv)
   free(authstatus);
   free(encmessage);
   free(encstatus);
-
   free(tempptr);
+
   return TCL_OK;
 }
  
@@ -760,8 +822,8 @@ int ui_find_keyname_by_key(dummy, interp, argc, argv)
 int gen_new_data(char *adstr,
                  char *new_data, 
                  char *keyname,
-   struct advert_data *addata )
-
+                 struct advert_data *addata,
+                 int addr_fam)
 {
     struct sap_header *bp=NULL;
     struct auth_info *authinfo=NULL;
@@ -769,9 +831,11 @@ int gen_new_data(char *adstr,
 
     char *buf=NULL;
 
+    int sap_hdr_len = 0;
     int newlen;
     int auth_len=0, hdr_len=0, privlen=0;
     int i;
+    u_int host;
 
     writelog(printf(" -- entered gen_new_data --\n");)
 
@@ -779,9 +843,17 @@ int gen_new_data(char *adstr,
 
     authinfo = addata->authinfo;
 
-    if (addata->sapenc_p != NULL) {
+  if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      sap_hdr_len = SAPV6_HDR_LEN;
+#endif
+  } else {
+      sap_hdr_len = SAPV4_HDR_LEN;
+  }
+
+  if (addata->sapenc_p != NULL) {
       sapenc_p = addata->sapenc_p;
-    }
+  }
 
 /* authPGPC and authX509C are obsolete and will be removed */
 
@@ -833,50 +905,59 @@ int gen_new_data(char *adstr,
 /* clear: addata->encrypt = 0, sapenc_p == NULL */
 
     if ( (addata->encrypt == 0) && (sapenc_p != NULL) ) {          /* asymm */
-      newlen=sizeof(struct sap_header)+TIMEOUT+hdr_len;
+      newlen=sap_hdr_len+TIMEOUT+hdr_len;
     } else if ( (addata->encrypt == 1) && (sapenc_p == NULL)) {    /*  symm */
-      newlen=sizeof(struct sap_header)+TIMEOUT+addata->length+4;
+      newlen=sap_hdr_len+TIMEOUT+addata->length+4;
     } else {                                                       /* clear */
-      newlen=sizeof(struct sap_header)+addata->length;
+      newlen=sap_hdr_len+addata->length;
     }
 
-/* create a new buffer and sap_header */
+/* create a new buffer and sapv4_header */
     
     buf = (char *)malloc(newlen);
-    bp  = (struct sap_header *)malloc(sizeof(struct sap_header));
-
+    bp  = (struct sap_header *)malloc(sap_hdr_len);
+    memset(bp, 0, sap_hdr_len);
     bp->version  = 1;
     bp->type     = 0;
     bp->compress = 0;
     bp->authlen  = auth_len/4;
     bp->msgid    = 0;
-    bp->src      = htonl(hostaddr);
-
+    /*bp->src      = htonl(hostaddr);*/
+  if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      memcpy((char *)bp+sizeof(struct sap_header), (char *) &hostaddr_v6, 16);
+      bp->addr     = 1;
+#endif
+  } else {
+      host = (unsigned long)htonl(hostaddr);
+      memcpy((char *)bp+sizeof(struct sap_header), (char *) &host, 4);
+      bp->addr     = 0;
+  }
 /* asymmetric, symmetric and then clear */
-/* copy the sap_header, priv_header and payload to buf */
+/* copy the sapv4_header, priv_header and payload to buf */
 
     if ( (addata->encrypt == 1) || (sapenc_p != NULL) ) { 
       bp->enc = 1;
-      memcpy(buf,bp,sizeof(struct sap_header));
+      memcpy(buf,bp,sap_hdr_len);
       free(bp);
 
 /* timeout field - always 0 at the moment */
 
     for (i=0; i<4; i++) {
-      buf[sizeof(struct sap_header)+i]=0;
+      buf[sap_hdr_len+i]=0;
     }
 
 /* add privacy header and data - 0 as we don't want an authentication header */
     
-      privlen = add_privacy_header(buf,0,sapenc_p);
+      privlen = add_privacy_header(buf,0,sapenc_p, addr_fam);
 
 /* add data directly after generic hdr for asymm and after padded for symm */
 
       if (sapenc_p != NULL) {                                    /* asymm */
-        memcpy(buf+sizeof(struct sap_header)+TIMEOUT+ENC_HEADER_LEN,
+        memcpy(buf+sap_hdr_len+TIMEOUT+ENC_HEADER_LEN,
                 addata->data,addata->length);
       } else {                                                   /*  symm */
-        memcpy(buf+sizeof(struct sap_header)+TIMEOUT+privlen,
+        memcpy(buf+sap_hdr_len+TIMEOUT+privlen,
                 addata->data,addata->length);
       }
       
@@ -885,9 +966,9 @@ int gen_new_data(char *adstr,
 /* no encryption was used so just copy the sap_header and the payload */
 
       bp->enc = 0;
-      memcpy(buf,bp,sizeof(struct sap_header));
+      memcpy(buf,bp,sap_hdr_len);
       free(bp);
-      memcpy(buf+sizeof(struct sap_header), addata->data, addata->length);   
+      memcpy(buf+sap_hdr_len, addata->data, addata->length);   
     }
 
 /* free up some internal variables */
@@ -910,42 +991,57 @@ int gen_new_data(char *adstr,
 /* ------------------------------------------------------------------- */
 int gen_new_auth_data(char *buf, 
                       char *new_data,
-         struct sap_header *bp,
+                      struct sap_header *bp,
                       int  auth_len,
-                      int  len)
+                      int  len,
+					  int addr_fam)
 {
   struct sap_header *bp1=NULL;
   char *newbuf=NULL, *sbuf=NULL;
   int newlen, lenrest;
+  int sap_hdr_len = 0;
 
   writelog(printf("entered gen_new_auth_data\n");)
 
 /* copy bp to bp1, setting bp->msgid=0 */
-
-  bp1 = (struct sap_header *)malloc(sizeof(struct sap_header));
+  if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      sap_hdr_len = SAPV6_HDR_LEN;
+#endif
+  } else {
+      sap_hdr_len = SAPV4_HDR_LEN;
+  }
+  bp1 = (struct sap_header *)malloc(sap_hdr_len);
+  memset(bp1, 0, sap_hdr_len);
   bp1->version  = bp->version;
   bp1->type     = bp->type;
   bp1->enc      = bp->enc;
   bp1->compress = bp->compress;
   bp1->authlen  = bp->authlen;
-  bp1->src      = bp->src;
   bp1->msgid    = 0;
+  bp1->addr     = bp->addr;
 
-/* calculate the length without the authentication header and malloc buffer */
+  if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      memcpy((char *)bp1+sizeof(struct sap_header), (char *)bp+sizeof(struct sap_header), 16);
+#endif
+  } else {
+      memcpy((char *)bp1+sizeof(struct sap_header), (char *)bp+sizeof(struct sap_header), 4);
+  }
 
   newlen = len - auth_len;
   newbuf = (char *)malloc(newlen);
 
-/* copy sap_header to start of newbuf and free temporary sap_header */
+/* copy sapv4_header to start of newbuf and free temporary sapv4_header */
 
-  memcpy((char *)newbuf,(char *)bp1,sizeof(struct sap_header));
+  memcpy((char *)newbuf,(char *)bp1,sap_hdr_len);
   free(bp1);
 
 /* copy everything else, missing out the auth header */
 
-  lenrest = newlen - sizeof(struct sap_header);
-  sbuf = (char *)buf+sizeof(struct sap_header)+auth_len;
-  memcpy((char *)newbuf+sizeof(struct sap_header),(char *)sbuf,lenrest);
+  lenrest = newlen - sap_hdr_len;
+  sbuf = (char *)buf+sap_hdr_len+auth_len;
+  memcpy((char *)newbuf+sap_hdr_len,(char *)sbuf,lenrest);
 
 /* copy the whole lot to the new_data buffer and free the work buffer */
 
@@ -972,9 +1068,13 @@ int write_authentication(char *afilename,char *data, int len, char *advertid)
   char tmpfilename[MAXFILENAMELEN];
   char *buf=NULL;
   char *tmpbuf=NULL, *newbuf=NULL;
+  char origsrc[128]="";
 
   int total,newlen,len1;
   int i=0, auth_len=0;
+  int sap_hdr_len = SAPV4_HDR_LEN;
+  int addr_fam=IPv4;
+  u_int host;
 
 #ifdef WIN32  /* need to sort out the ~ on windows */
   struct stat sbuf;
@@ -986,6 +1086,14 @@ int write_authentication(char *afilename,char *data, int len, char *advertid)
 
   writelog(printf(" -- entered write_authentication --\n");)
 
+  sscanf(&data[2], "%s", origsrc);
+  if (strstr(origsrc,".") == NULL) {/* Checking for an IPv6 address */
+#ifdef HAVE_IPv6
+      sap_hdr_len = SAPV6_HDR_LEN;
+	  addr_fam=IPv6;
+#endif
+  }
+
   addata = get_advert_info(advertid);
 
   if (addata  == NULL) {
@@ -1023,10 +1131,10 @@ int write_authentication(char *afilename,char *data, int len, char *advertid)
 
 /* malloc the main buffer */
 
-  total = len + sizeof(struct sap_header) + auth_len + newlen;
+  total = len + sap_hdr_len + auth_len + newlen;
   buf = (char *)malloc(total);
 
-  writelog(printf("write_auth: malloced: len(%d) + sap_hdr(%d) + auth_len(%d) + newlen(%d) = total(%d)\n",len,sizeof(struct sap_header),auth_len,newlen,total);)
+  writelog(printf("write_auth: malloced: len(%d) + sap_hdr(%d) + auth_len(%d) + newlen(%d) = total(%d)\n",len,sap_hdr_len,auth_len,newlen,total);)
 
 /* write the data (starting "n=...v=...Z=\n"to the buffer */
 
@@ -1037,25 +1145,36 @@ int write_authentication(char *afilename,char *data, int len, char *advertid)
 
 /* write the sap header into the buffer                            */
 
-  sap_hdr = addata->sap_hdr;
+  sap_hdr = (struct sap_header *)addata->sap_hdr;
 
   if( sap_hdr == NULL) {
-
-    sap_hdr = (struct sap_header *)malloc(sizeof(struct sap_header));
+    sap_hdr = (struct sap_header *)malloc(sap_hdr_len);
+	memset(sap_hdr, 0, sap_hdr_len);
     sap_hdr->version  = 1;
     sap_hdr->authlen  = auth_len /4;	
     sap_hdr->enc      = 0;
     sap_hdr->compress = 0;
     sap_hdr->msgid    = 0;
-    sap_hdr->src      = htonl(hostaddr);
+    /*sap_hdr->src      = htonl(hostaddr);*/
 
-    memcpy(buf+len,sap_hdr,sizeof(struct sap_header));
-    len += sizeof(struct sap_header);
+    if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      memcpy((char *)sap_hdr+sizeof(struct sap_header), (char *) &hostaddr_v6, 16);
+      sap_hdr->addr     = 1;
+#endif
+	} else {
+      host = (unsigned long)htonl(hostaddr);
+      memcpy((char *)sap_hdr+sizeof(struct sap_header), (char *) &host, 4);
+      sap_hdr->addr     = 0;
+	}
+
+    memcpy(buf+len,sap_hdr,sap_hdr_len);
+    len += sap_hdr_len;
     free(sap_hdr);
 
   } else {
-    memcpy(buf+len,sap_hdr,sizeof(struct sap_header));
-    len += sizeof(struct sap_header);
+    memcpy(buf+len,sap_hdr,sap_hdr_len);
+    len += sap_hdr_len;
   }
 
   if (auth_len !=0) {
@@ -1127,6 +1246,8 @@ int write_authentication(char *afilename,char *data, int len, char *advertid)
 #else
   rename(tmpfilename, filename);
 #endif
+
+/* endif - MM ifndef HAVE_IPv6 */
   return 0;
 }
 
@@ -1147,11 +1268,15 @@ int write_encryption(char *afilename, char *data, int len , char *auth_type, cha
   char *filename;
   char tmpfilename[MAXFILENAMELEN];
   char *buf=NULL;
+  char origsrc[128]="";
   int i=0, auth_len=0;
   int hdr_len=0;
   int orglen;
   int packetlength=0;
   struct advert_data *get_advert_info();
+  int sap_hdr_len = SAPV4_HDR_LEN;
+  int addr_fam=IPv4;
+  u_int host;
 
 #ifdef WIN32  /* need to sort out the ~ on windows */
   struct stat sbuf;
@@ -1163,6 +1288,14 @@ int write_encryption(char *afilename, char *data, int len , char *auth_type, cha
 
   writelog(printf(" -- entered write_encryption (filename = %s)\n",afilename);)
 
+  sscanf(&data[2], "%s", origsrc);
+  if (strstr(origsrc,".") == NULL) {/* Checking for an IPv6 address */
+#ifdef HAVE_IPv6
+      sap_hdr_len = SAPV6_HDR_LEN;
+	  addr_fam=IPv6;
+#endif
+  }
+
 /* get the information for the advert */
 
   addata = get_advert_info(advertid);
@@ -1195,12 +1328,12 @@ int write_encryption(char *afilename, char *data, int len , char *auth_type, cha
 /* malloc the output buffer - not we have the plain data followed by the */
 /* full SAP packet                                                       */
 
-  packetlength = sizeof(struct sap_header)+auth_len+TIMEOUT+hdr_len;
+  packetlength = sap_hdr_len+auth_len+TIMEOUT+hdr_len;
   buf = (char *)malloc(len+packetlength);
 
 /* debug */
 
-  writelog(printf("write_enc: malloced: len(%d) + sizeof sap_hdr(%d) + auth_len(%d) + T/O(%d) + hdr_len(%d) = %d\n",len,sizeof(struct sap_header),auth_len,TIMEOUT,hdr_len,(len+sizeof(struct sap_header)+auth_len+TIMEOUT+hdr_len));)
+  writelog(printf("write_enc: malloced: len(%d) + sizeof sap_hdr(%d) + auth_len(%d) + T/O(%d) + hdr_len(%d) = %d\n",len,sap_hdr_len,auth_len,TIMEOUT,hdr_len,(len+sap_hdr_len+auth_len+TIMEOUT+hdr_len));)
 
 /* copy plain data to the buffer */
 
@@ -1208,28 +1341,39 @@ int write_encryption(char *afilename, char *data, int len , char *auth_type, cha
 
 /* set up sap header */
 
-  bp = addata->sap_hdr;
+  bp = (struct sap_header *)addata->sap_hdr;
 
   if (bp == NULL) {
-    bp=malloc(sizeof(struct sap_header));
+    bp=malloc(sap_hdr_len);
+	memset(bp, 0, sap_hdr_len);
     bp->version  = 1;
     bp->authlen  = auth_len /4;	
     bp->enc      = 1;
     bp->compress = 0;
     bp->msgid    = 0;
-    bp->src      = (unsigned long)htonl(hostaddr);
-  }
+    /*bp->src      = (unsigned long)htonl(hostaddr);*/
 
+    if (addr_fam == IPv6) {
+#ifdef HAVE_IPv6
+      memcpy((char *)bp+sizeof(struct sap_header), (char *) &hostaddr_v6, 16);
+      bp->addr     = 1;
+#endif
+	} else {
+      host = (unsigned long)htonl(hostaddr);
+      memcpy((char *)bp+sizeof(struct sap_header), (char *) &host, 4);
+      bp->addr     = 0;
+	}
+  }
 /* debug */
 
   writelog(printf("write_enc: bp: version=%d type=%d enc=%d compress=%d authlen=%d msgid=%d src=%lu\n",bp->version, bp->type, bp->enc, bp->compress, bp->authlen, bp->msgid, bp->src);)
 
 /* copy sap header to the buffer */
 
-  memcpy((char *)buf+len,(char *)bp,sizeof(struct sap_header));
+  memcpy((char *)buf+len,(char *)bp,sap_hdr_len);
 
   orglen = len;
-  len += sizeof(struct sap_header);
+  len += sap_hdr_len;
 
 /* copy authentication info to the buffer */
 
@@ -1262,7 +1406,7 @@ int write_encryption(char *afilename, char *data, int len , char *auth_type, cha
 /* Now add the timeout field - always 0 at the moment      */
 
   for (i=0; i<4; i++) {
-    buf[orglen+sizeof(struct sap_header)+auth_len+i]=0;
+    buf[orglen+sap_hdr_len+auth_len+i]=0;
   }
   len += TIMEOUT;
 
@@ -1307,5 +1451,8 @@ int write_encryption(char *afilename, char *data, int len , char *auth_type, cha
 #else
   rename(tmpfilename, filename);
 #endif
+
+/* #endif - MM ifndef HAVE_IPv6 */
   return 0;
+
 }
