@@ -70,7 +70,9 @@
 #define GUI 1
 #define NO_GUI 0
 
-
+#ifndef INET_ADDRSTRLEN
+#define INET_ADDRSTRLEN (16)
+#endif
 
 #ifdef WIN32
 
@@ -117,10 +119,25 @@ time_t time(time_t *);
 char *strerror(int i);
 #endif
 
+#define IPv4 4
+#define IPV4_ADDR_LEN 4
+#define IPv6 6
+#ifdef HAVE_IPv6
+#include "ipv6_macros.h"
+#endif
+
+
 #define SAP_GROUP        "224.2.127.254"
 #define SAP_PORT         9875
+
+#define SAPv6_DEFAULT_MASK 13
+#define SAPv6_GROUP        "ff0e:0:0:0:0:0:2:7ffe"
+#define SAPv6_DEFAULT      "ff0e::2:8000"
+
 #define SIP_GROUP        "224.0.1.75"
-#define SIP_PORT        5060
+#define SIP_PORT         5060
+#define SIPv6_GROUP      "ff0e:0:0:0:0:0:0:014b"
+
 
 
 #ifdef TRUE
@@ -168,6 +185,7 @@ struct advert_data {
   char *aid;
   char *data;
   int tx_sock;
+  int addr_fam;
 #ifdef AUTH
   struct auth_info *authinfo;
   struct sap_header *sap_hdr;
@@ -183,22 +201,74 @@ struct advert_data {
 };
   
 struct sap_header {
-#ifdef DIFF_BYTE_ORDER
-  u_int compress:1;
-  u_int enc:1;
-  u_int type:3;
+#ifdef WORDS_BIGENDIAN
   u_int version:3;
+  u_int addr:1;
+  u_int resv:1;
+  u_int type:1;
+  u_int enc:1;
+  u_int compress:1;
 #else
-  u_int version:3;
-  u_int type:3;
-  u_int enc:1;
   u_int compress:1;
+  u_int enc:1;
+  u_int type:1;
+  u_int resv:1;
+  u_int addr:1;
+  u_int version:3;
 #endif
   u_int authlen:8;
   u_int msgid:16;
-  u_int src;
+/* MM -make protocol independent, see new sap_header def's below */
+/*  u_int src; */
 };
 
+/* These repitious sap headers are UGLY... */
+struct sapv4_header {
+#ifdef WORDS_BIGENDIAN
+  u_int version:3;
+  u_int addr:1;
+  u_int resv:1;
+  u_int type:1;
+  u_int enc:1;
+  u_int compress:1;
+#else
+  u_int compress:1;
+  u_int enc:1;
+  u_int type:1;
+  u_int resv:1;
+  u_int addr:1;
+  u_int version:3;
+#endif
+  u_int authlen:8;
+  u_int msgid:16;
+  u_int src; 
+};
+
+struct sapv6_header {
+#ifdef WORDS_BIGENDIAN
+  u_int version:3;
+  u_int addr:1;
+  u_int resv:1;
+  u_int type:1;
+  u_int enc:1;
+  u_int compress:1;
+#else
+  u_int compress:1;
+  u_int enc:1;
+  u_int type:1;
+  u_int resv:1;
+  u_int addr:1;
+  u_int version:3;
+#endif
+  u_int authlen:8;
+  u_int msgid:16;
+  unsigned char src[16];
+};
+
+
+#define SAPV6_HDR_LEN sizeof(struct sapv6_header)
+#define SAPV4_HDR_LEN sizeof(struct sapv4_header)
+
 /*we use this to store info about authentication*/
 struct auth_info {
   u_int auth_type;
@@ -214,14 +284,14 @@ struct auth_info {
 
 /*this is the actual header that goes in the packets*/
 struct auth_header {
-#ifdef DIFF_BYTE_ORDER
-  u_int auth_type:4;
-  u_int padding:1;
+#ifdef WORDS_BIGENDIAN
   u_int version:3;
+  u_int padding:1;
+  u_int auth_type:4;
 #else
-  u_int version:3;
-  u_int padding:1;
   u_int auth_type:4;
+  u_int padding:1;
+  u_int version:3;
 #endif
   u_int siglen:8;
 };
@@ -230,14 +300,14 @@ struct auth_header {
 #define ENC_HEADER_LEN  2
 
 struct priv_header {
-#ifdef DIFF_BYTE_ORDER
-  u_int enc_type:4;
-  u_int padding:1;
+#ifdef WORDS_BIGENDIAN
   u_int version:3;
+  u_int padding:1;
+  u_int enc_type:4;
 #else
-  u_int version:3;
-  u_int padding:1;
   u_int enc_type:4;
+  u_int padding:1;
+  u_int version:3;
 #endif
   u_int hdr_len:8;
   u_int pad_len;
