@@ -35,12 +35,19 @@
 #include "ui_fns.h"
 #include "prototypes.h"
 
+#ifdef HAVE_IPv6
+#include "ipv6_macros.h"
+#endif
+
 extern Tcl_Interp *interp;
 
 extern char hostname[];
 extern char username[];
 extern char sipalias[];
 extern unsigned long hostaddr;
+#ifdef HAVE_IPv6
+struct in6_addr hostaddr_v6;
+#endif
 extern int rxsock[];
 extern int no_of_rx_socks;
 extern int doexit;
@@ -51,7 +58,8 @@ int ui_sd_listen(dummy, interp, argc, argv)
     int argc;                           /* Number of arguments. */
     char **argv;                        /* Argument strings. */
 {
-  sd_listen(argv[1], atoi(argv[2]), rxsock, &no_of_rx_socks, 1);
+  sd_listen(argv[1], atoi(argv[2]), atoi(argv[3]), rxsock, &no_of_rx_socks, 1);
+  sprintf(interp->result, "%d", 4);
   return TCL_OK;
 }
 
@@ -85,6 +93,21 @@ int ui_lookup_host(dummy, interp, argc, argv)
   return TCL_OK;
 }
 
+int ui_verify_ipv6_stack(dummy, interp, argc, argv)
+    ClientData dummy;                   /* Not used. */
+    Tcl_Interp *interp;                 /* Current interpreter. */
+    int argc;                           /* Number of arguments. */
+    char **argv;                        /* Argument strings. */
+{
+
+#if defined(HAVE_IPv6)
+        sprintf(interp->result, "%d", 1);
+#else 
+        sprintf(interp->result, "%d", 0);
+#endif
+    return TCL_OK;
+}
+
 int ui_sip_send_udp(dummy, interp, argc, argv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
@@ -161,7 +184,7 @@ int ui_check_address(dummy, interp, argc, argv)
 {
   struct in_addr in;
   in.s_addr=inet_addr(argv[1]);
-  if(check_address(&in)==TRUE)
+  if(check_address(&in, IPv4)==TRUE)
     {
       interp->result="ok";
     }
@@ -189,6 +212,35 @@ int ui_generate_address(dummy, interp, argc, argv)
   return TCL_OK;
 }
 
+int ui_generate_v6_address(dummy, interp, argc, argv)
+    ClientData dummy;                   /* Not used. */
+    Tcl_Interp *interp;                 /* Current interpreter. */
+    int argc;                           /* Number of arguments. */
+    char **argv;                        /* Argument strings. */
+{
+
+#ifdef HAVE_IPv6
+  struct in6_addr in;
+  struct in6_addr in2;
+
+  /* printf("entering ui_generate_v6_address: no of args: %d,  %s %d\n", 
+         argc,argv[1],atoi(argv[2])); /* */
+  
+  if (argc==1) {
+      printf("ui_generate_v6_address: no args???\n");
+      IN6_pADDR_COPY(&in, generate_v6_address(NULL, 0, NULL));
+  } else {
+      inet_pton(AF_INET6, argv[1], &in);
+      generate_v6_address(&in, atoi(argv[2]), &in2);
+  }   
+  
+  inet_ntop(AF_INET6, &in2, interp->result, INET6_ADDRSTRLEN);
+  /*printf("exiting ui_generate_v6_address: result%s\n", interp->result); /* */
+#endif /* HAVE_IPv6 */
+
+  return TCL_OK;
+}
+
 static char daynames[7][4];
 static char longdaynames[7][21];
 static char monnames[12][4];
@@ -266,6 +318,7 @@ int ui_getdayname(dummy, interp, argc, argv)
   return TCL_OK;
 }
 
+
 int ui_getmonname(dummy, interp, argc, argv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
@@ -315,6 +368,19 @@ int ui_gethostaddr(dummy, interp, argc, argv)
   return TCL_OK;
 }
 
+int ui_get_v6_hostaddr(dummy, interp, argc, argv)
+    ClientData dummy;                   /* Not used. */
+    Tcl_Interp *interp;                 /* Current interpreter. */
+    int argc;                           /* Number of arguments. */
+    char **argv; 
+{
+#ifdef HAVE_IPv6
+  inet_ntop(AF_INET6, &hostaddr_v6, interp->result, INET6_ADDRSTRLEN);
+#endif
+  return TCL_OK;
+}
+
+
 int ui_gethostname(dummy, interp, argc, argv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
