@@ -173,11 +173,26 @@ ui_init(int *argc, char **argv)
 int announce_error(int code, char *command)
 {
 	if (code != TCL_OK) {
-		char buf[128];
-		strncpy(buf, interp->result, sizeof(buf) - 1);
-		fprintf(stderr, "sdr:%s %s\n", command, interp->result);
-		Tcl_VarEval(interp, "puts $errorInfo", NULL);
-		Tcl_VarEval(interp, "tkerror {", buf, "}", NULL);
+                char buf[128];
+#ifdef WIN32
+                DWORD dwMBResponse;
+                char *szFmt="Sdr failed with the following error:\n\n%s %s\n\nPlease report to sdr@cs.ucl.ac.uk.\n\nContinue running application?", *szErrMsg;   
+                DWORD dwErrMsgLen = strlen(command) + strlen(interp->result) + strlen(szFmt);
+                
+                szErrMsg = (char*)malloc(dwErrMsgLen);
+                sprintf(szErrMsg, szFmt, command, interp->result);
+                dwMBResponse = MessageBox(NULL, szErrMsg, "SDR Error", MB_ICONERROR|MB_YESNO);
+                free(szErrMsg);
+                if (dwMBResponse == IDNO) {
+                        exit(-1);
+                }       
+#else
+                fprintf(stderr, "sdr:%s %s\n", command, interp->result);
+#endif
+                Tcl_VarEval(interp, "puts $errorInfo", NULL);
+                strncpy(buf, interp->result, sizeof(buf) - 1);
+                buf[sizeof(buf) - 1] = 0; /* Let's not overrun */
+        	Tcl_VarEval(interp, "tkerror {", buf, "}", NULL);	        
 	}
 	return (code);
 }
@@ -190,8 +205,7 @@ int ui_create_interface()
    * Set the geometry of the main window, if requested.
    */
   if (geometry != NULL) {
-    announce_error(Tcl_VarEval(interp, "wm geometry . ", geometry, 0), 
-		   "setting geometry");
+    Tcl_SetVar(interp, "geometry", geometry, TCL_GLOBAL_ONLY);
   }
 
   for(i=0;i<MAX_TCL_MODULE;i++)
