@@ -36,7 +36,7 @@
 
 #include "sdr.h"
 
-#define close closesocket
+/* #define close closesocket */
 
 int
 uname(struct utsname *ub)
@@ -155,6 +155,7 @@ WinMain(
     HINSTANCE hPrevInstance,
     LPSTR lpszCmdLine,
     int nCmdShow)
+
 {
     char *p;
     WSADATA WSAdata;
